// MODEL 203 DATA ACQUISITON SYSTEM 

// Lawson Labs, Inc.
// 3239 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800
// December 16, 1997

//*************************************************************************//
//  GLOBAL VARIABLES
//
// Standard deviation calculation variables
double	Max=-6000;			// Maximum Voltage since last cleared
double 	Min=6000;			// Minimum Voltage since last cleared
double	stDv=0;				// Standard Deviation of voltage
double 	rmsBits=0;      	// System Noise expressed in Bits
double	sdTot = 0;       	// Total of voltages included in SD calculation  	
double 	Average =0;			// Average of voltages included in SD calculation
double	sd[41];				// Array for voltages included in SD calculation
int		sdCount=1;			// Number of voltages included in each SD calculation

// Configuration File access variables
char	InputString[80];	// Input string array used for reading config file
char	OutputString[80];	// Output string array used for reading config file
char	*ISptr;         	// Input string array pointer
char	*OSptr;				// Output string array pointer
int		reserved;			// Dummy variables for unused  
int		spare;
int		placeholder;

// Data Logging File variables
char	FileName[12]="<none>   ";		// Log file name 		
char	NewFileName[12];	// Requested Log file name
FILE	*LogFile;			// Log file handle 
BOOLEAN fLogToFile = FALSE;	// Set to TRUE when logging data, FALSE = no logging 

//Scanning Mode variables
BYTE	ScanToken=0x8B;		// Default Scanning Mode command token
BYTE	Edge=0x00;			// Default (positive) Triggered Scanning Edge
int		ScanMode;			// Active scanning mode - Normal, Triggered, etc.
//6-21-06 int		ScanError;			// Scanning Error code
short		ScanError;			// Scanning Error code
char	scandisp;			// Scanning display mode Text or Graphics
int		posfs,negfs,vdiv,hdiv; // Graphics display parameters 
	
// Voltage Array varibles
double	Voltage;         	// Last calculated voltage
short	ChanChgFlg;    		// Channel Change Flag, 1 = Channel has been changed
double	CountTotal;    		// Running voltage average array total
short	ArrayPointer;   	// Running voltage average array pointer
double   CountArray[100];  // Running voltage average array

// Table of BAUD rates (0 - 5)
short    bauds[]={9600,4800,2400,1200,600,300};

BYTE fDebug = 0;
BYTE AppNewScanBuff[300];
BYTE* pAppNewScanBuff;
char BuffDebugMsg[255];
//
//*************************************************************************//
//  M203 VARIABLES
//
//  		This structure defines variables shared by this program (EasyWin)
//		and the Model 203 DLL (16 or 32 bit). 
//
typedef struct{
	short	comPort;		// Currently selected comm port (0 to 3) for COM1 .. COM4
	double	scanInterval;	// Current scan interval in seconds
	short	numChannels;	// Number of scanning channels (always 1 for Model 203)
	short	tScan;			// Scan type (always 0 for Model 203)
	short	scanChan;		// Current scan mode channel (0,1,6, or 7)
	short	lastDigInp;		// Last digital input value
	short 	version;			// Model 203 version (203)
	short 	pollChan;			// Current polled mode channel
	short 	subSample;		// Number of subsamples per conversion 160 for 60HZ, 192 for 50 Hz
	unsigned long rawCount;	// Last raw count returned from Model 203
	int		digOutWord;		// Last value sent to Model 203 for digital output (0-255)
	short	wordCnt;			// number of bytes in a conversion result (2 = 16-bit, 3 = 24-bit)
	int		errorType;		// error code returned from DLL
	int		bU;				// BiPolar UniPolar Input Range (Not Used on Model 203)
	short 	filter;			// Input filter selection (not used on Model 203)      
	short 	divideBy;		// Remote (power of 2) division (0 to 7)
	double 	gain;				// Last calculated Gain
	float 	maxRate;			// Maximum sample rate (120 Hz for 60Hz, 100 Hz for 50Hz)
	short 	totalPeriods;	// (modeRegLo) - Number of conversions per sample
	short 	modeRegLo;		//
	short 	modeRegMid;		// MSB = Word Length, always 1 (24 bit), remaining bits unused 
	short 	modeRegHi;		// Mode Register High - Unused on Model 203
	short	baud;				// BAUD rate code (0-5) 0=9600 bps ... 5=300 bps
	float	dac1volts;		// Current Voltage for Analog Output 1
	float	dac2volts;    	// Current Voltage for Analog Output 2
	short	dacEnable;    	// Not Used (DACs always enabled)
	int		timeBase;  	  	// must be a positive value
	long	echoTestCount; // Serial Echo Test Byte Count
	unsigned long 	offsetTotal;	// Last calculated Offset
	unsigned long 	runningAvg;		// Running Average
	short		dataAcqMode;	// 0 = Scanning, 1 = Polled
	unsigned int inputBuffer;	//receive queue sizes for Windows COMM driver
	short	wL;          	// Word Length - Always 128 for 24-bit with M203
	float	gain1;         // Last calculated Gain
	double	totalBits;		// Total possible bits of resolution
	//char	scan;        	// 
	BYTE	scan;
	//char	scan1;        	//
	BYTE	scan1;
	//char	scan2;       	//
	BYTE	scan2;
	int		retry;			// Number of SignOn retrys
	short	ChanChgFlg;
	double	Voltage;         	// Last calculated voltage
}	T_M203;

//*************************************************************************//
// LOCAL FUNCTION PROTOTYPES
//  (See individual routine headers for descriptions)
short	GetVoltage_LOCAL(double *Voltage);
short	ReadConfig(void);
short	WriteConfig(void);
short	ScanMenu(void);
void 	Wait(float milliseconds);		
void 	PrintPolledScreen(void);	
void 	DoCommand(int keyinput);   
void 	ZeroDelta(void);
void 	ClearSD(void);
void 	ClearCommandLines(void);
void 	PrintTopMenuScreen();
void 	SetWindowToDesktop(void);
void 	SetWin32CursorOn(void);
void 	SetWin32CursorOff(void);
void	SelectRate(void);
void	StandardDev(double Voltage,double *stDv, double *rmsBits, 
			double *sdTot, double *sd, double *Average, int *sdCount);
void 	StringSeg(int PIndex, char *ISptr, char *OSprt);


BOOLEAN	LoadLibraries(void);
BOOLEAN	CheckLibFuncs(void);
BOOLEAN	UnloadLibraries(void);

HANDLE idComDev;


//==========================================================================
// DLL FUNCTION PROTOTYPES
// Note that their are full descriptions of each of the functions at the 
//		end of this header file. 
//==========================================================================

//short	ClearVoltageArray(void);

typedef UINT(*DLL_GET_NUM_SCANS_IN_BUFFER)(void);
DLL_GET_NUM_SCANS_IN_BUFFER GetNumScansInBuffer;

typedef void(*DLL_GCONV)(void);
DLL_GCONV Gconv;

typedef short(*DLL_GET_VOLTAGE)(void);
DLL_GET_VOLTAGE GetVoltage;



typedef short(*DLL_CLEAR_VOLTAGE_ARRAY)(void);
DLL_CLEAR_VOLTAGE_ARRAY ClearVoltageArray;

//short	SetPtrs(T_M203 *M203, BYTE*);
typedef short(*DLL_SET_PTRS)(T_M203 *M203, BYTE* pAppNewScanBuff, char* BuffDebugMsg );
DLL_SET_PTRS SetPtrs;

//short	SignOn(short EchoTestFlg, BYTE fDebug);
typedef short(*DLL_SIGN_ON)(short EchoTestFlg, BYTE fDebug);
DLL_SIGN_ON SignOn;

//short	FullScaleCal(void);
typedef short(*DLL_FULL_SCALE_CAL)(void);
DLL_FULL_SCALE_CAL FullScaleCal;

//short	OffsetCal(void);
typedef short(*DLL_OFFSET_CAL)(void);
DLL_OFFSET_CAL OffsetCal;

//short	SysCal(void);
typedef short(*DLL_SYS_CAL)(void);
DLL_SYS_CAL SysCal;

//short	GetDigIn(int token);
typedef short(*DLL_GET_DIG_IN)(int token);
DLL_GET_DIG_IN GetDigIn;

//void	SetDigOut(void);
typedef short(*DLL_SET_DIG_OUT)(void);
DLL_SET_DIG_OUT SetDigOut;

//short	GetVer(void);
typedef short(*DLL_GET_VER)(void);
DLL_GET_VER GetVer;

//short	GetConversion(void);
typedef short(*DLL_GET_CONVERSION)(void);
DLL_GET_CONVERSION GetConversion;

//void	SetChanCode(void);
typedef short(*DLL_SET_CHAN_CODE)(void);
DLL_SET_CHAN_CODE SetChanCode;

//short	StartScanning(void);
typedef short(WINAPI *DLL_START_SCANNING)(int ScanToken, int Edge);
DLL_START_SCANNING StartScanning;

//short	GetOneScan(void);
typedef short(*DLL_GET_ONE_SCAN)(void);
DLL_GET_ONE_SCAN GetOneScan;

//short	StopScanning(void);
typedef short(*DLL_STOP_SCANNING)(void);
DLL_STOP_SCANNING StopScanning;

//short	SendADMode(void);
typedef short(*DLL_SEND_AD_MODE)(void);
DLL_SEND_AD_MODE SendADMode;

//short	GetCheckSum(void);
typedef short(*DLL_GET_CHECKSUM)(void);
DLL_GET_CHECKSUM GetCheckSum;

//void	CloseDevice(void);
typedef short(*DLL_CLOSE_DEVICE)(void);
DLL_CLOSE_DEVICE CloseDevice;

//short	ChangeAnalogOutput(short DacToSet);
typedef short(*DLL_CHANGE_ANALOG_OUTPUT)(short DacToSet);
DLL_CHANGE_ANALOG_OUTPUT ChangeAnalogOutput;

//short	CheckScanTime(double *maxTime, double *minTime);
typedef short(*DLL_CHECK_SCAN_TIME)(double *NewScanInterval,double *maxTime, double *minTime);
DLL_CHECK_SCAN_TIME CheckScanTime;

//short	EchoTest(void);
typedef short(*DLL_ECHO_TEST)(void);
DLL_ECHO_TEST EchoTest;

//void	RemFromBuffer(short size);
typedef short(*DLL_REM_FROM_BUFFER)(short size);
DLL_REM_FROM_BUFFER RemFromBuffer;


//==========================================================================
// FULL DESCRIPTIONS OF DLL FUNCTION PROTOTYPES
//==========================================================================
/*
//==========================================================================
//  CLEARVOLTAGEARRAY - This function clears the voltage array that is used
//				for storing and averaging of data returned by the 203
short  ClearVoltageArray(void);

//==========================================================================
//  SETPTRS - This function sets any pointers to be used between the DLL
//			and instance of calling application. Especially important, is it
//			sets the pointer to the instance of the M203 structured data.
//       Note that the memory for the structure is established in the calling
//			calling application, and the DLL holds only a pointer to that memory
//
// Other Routines Called: None
// Output: None
// Return Values: BOOLEAN
//		TRUE = Success.
//		FALSE = Failed.
short  SetPtrs(T_M203 *M203, BYTE*);

//==========================================================================
//  SIGNON - This function establishes communications with the Model 203
//		using the settings in M203 configuration file and the
//		selected BAUD rate, comm Port, and input buffer size.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->baud, M203->comPort, M203->modeRegHi, M203->modeRegMid,
//			M203->totalPeriods, M203->timeBase, M203->divideBy, M203->scanInterval
//			M203->scan, M203->scan1, M203->scan2, M203->inputBuffer
//		EchoTestFlag - 1 for SignOn with EchoTest, 0 for normal SignOn
//			Note: If EchoTest is used, two calls to SignOn must be performed.
//			The first call executes the first portion of the signon procedure and
//			starts the serial byte echo test.  The second call stops the echo test
//			and completes the sign on procedure.
//
// Other Routines Called: None
// Output:
//		M203->retry, M203->echoTestCount
// Return Values:
//		0 = Successful sign on, port opened, Model 203 active.
//		1 = Unsuccessful signon, comm port is closed
//
short  SignOn(short EchoTestFlg, BYTE fDebug);

//==========================================================================
// FULLSCALECAL - Perform a full-scale calibration on the currently
//		selected channel, makes the currently selected channel read 5 volts.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->runningAvg, M203->offsetTotal, M203->rawCount, RANGE
//
// Other Routines Called: GetConversion
// Output: M203->FullScaleTotal, M203->gain, M203->totalBits
// Return Values:
// 	0 = Successful Full-Scale Calibration
//		1 = Count difference between the full-scale reading and the
//				last offset calibration reading is too small to produce
//				an acceptable calibration
//		2 = GetConversion Error
short  FullScaleCal(void);

//==========================================================================
// OFFSETCAL - Perform an offset calibration on the currently
//		selected channel.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->runningAvg, M203->rawCount
// Other Routines Called: GetConversion
// Output: M203->offsetTotal
// Return Values:
//  	0 = Succesful Offset calibration
//	 	1 = GetConversion Error
short  OffsetCal(void);

//==========================================================================
// SYSCAL - Perform a system calibration which consists of an offset calibration
//		on channel 7 followed by a full-scale calibration on channel 6. Once
//		calibration is completed, the active channel is reselected.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->scanChan, M203->pollChan
// Other Routines Called: OffsetCal, FullScaleCal
// Output:
// Return Values:
//		0 = No errors, a successfull system calibration was performed
//   	1 = Calibration error
short  SysCal(void);

//==========================================================================
// GETDIGIN - This routine instructs the Model 203 to read the
//		"RS" digital input.
//
// Input: token = Get Digital Input Command token
//			80 = Get digital input, 8E = Get digital input with checksum
// 		Argument = always 4Ch
// Other Routines Called: None
// Output: 	Bit 0 of M203->lastDigInp = 0 if "RS" input is low (0 Vdc),
//										        = 1 if "RS" input is high (5 Vdc)
// Return Values:
//		0 = Successfull input read
//	 	1 = Incorrect running checksum (8E digital checksum only)
//	 	2 = Incorrect GetDigitalInput command token echoed
//    3 = TimeOut waiting for command echo
short  GetDigIn(int token);

//==========================================================================
// SETDIGOUT - This function instructs the Model 203 to output the  
//		digital output code in M203->digOutWord to the digital output
//		port termials "A" through "H".
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->digOutWord
// Other Routines Called: None
// Output:
// Return Values: none
void  SetDigOut(void);

//==========================================================================
// Get Version - This function obtains the current version number from
//		the data acquisition system and places it into the M203 Variable
//		sturcture as M203->version.
//
// Input: none
// Other Routines Called: None
// Output: M203->version
// Return Values:
//   	0 = No Errors
//		1 = Timeout waiting for Version Number to be returned
//		2 = Incorrect command token echoed
//		3 = Timeout waiting for command token to be echoed
short  GetVer(void);

//==========================================================================
// GETCONVERSION - This routine gets one conversion data sample from the
//		Model 203 on the selected channel. The conversion result is placed
//		into M203->rawCount.  This function is used to obtain data samples
//		during polled mode and to perform calibrations.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->wordCnt
// Other Routines Called: None
// Output: M203->rawCount
// Return Values:
//   	0 = No Errors, new conversion obtained
//		1 = Incorrect get conversion token echoed
// 	2 = TimeOut waiting for GetConversion command token and data
//				OR communications port error
// 	3 = Error in sending conversion request
short	GetConversion(void);

//==========================================================================
// SETCHANCODE - This function instructs the Model 203 to select the
//		desired channel.  If scanning (M203->dataAcqMode = 0), M203->scanChan
//		is selected.  If polled mode (M203->dataAcqMode = 1), MM203->pollChan
//		is selected.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->dataAcqMode, M203->scanChan, M203->pollChan
// Other routines called: none
// Output: none
// Return Values: None
void	SetChanCode(void);

//==========================================================================
//	STARTSCANNING - This function instructs the Model 203 to start scanning
//		using the selected scanning mode.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->wordCnt, M203->scanInterval
//	  ScanToken		0x8B = Normal Scanning
//						0x8D = CheckSum Scanning
//						0x89 = Normal, Triggered Scanning
//						0x8C = Checksum, Triggered Scanning
//
//		Edge (required for triggered scanning only)
//							0 = Positive Edge Trigger
//               		!0 = Negative Edge Trigger
//
// Other Routines Called: None
// Output: None
// Return Values:
//		0 = Successfull scanning start
//   	1 = Incorrect command token echoed.
//   	2 = Time out waiting for command token echo.
short	StartScanning(void);

//==========================================================================
//	GETONESCAN - This function gets one scan from the input buffer. The
//		result is placed in M203->rawCount.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->inputBuffer, M203->scanInterval, M203->wordCnt
// Other Routines Called: None
// Output: M203->rawCount
// Return Values:
//   -1 = Time Out occurred waiting for scan (waiting for Trigger)
// 	0 = Scan received.
//   	1 = Time Out occurred waiting for scan
//   	2 = Input Buffer overflow detected.
short	GetOneScan(void);

//==========================================================================
//	STOPSCANNING - This function instructs the Model 203 to stop scanning and
//		empties the input buffer.
//
// Input: None
// Other Routines Called: None
// Output: None
// Return Values:
//		0 = Successfull, Scanning is discontinued.
//	 	1 = Stop Scan command token not echoed.
short	StopScanning(void);

//==========================================================================
// SEND ADMODE - Sets the Model 203 mode registers.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->modeRegHi, M203->modeRegLo, M203->modeRegLo,
//			M203->totalPeriods, M203->wL, M203->timeBase
// Other Routines Called: None
// Output:
// Return Values:
//		0 = Successfull, no Errors
//   	1 = mode registers echoed are incorrect
//		2 = Time out waiting for moderegisters to be echoed
//		3 = Incorrect command token echoed
//		4 = Time out waiting for command token echo
short	SendADMode(void);

//==========================================================================
// GET CHECKSUM - This function obtains and tests the Model 203 running
//		checksum.
//
// Other Routines Called: WaitChar, SendPacket
// Output - Global Variables: None
// 	RunningCheckSum = 0 - Anytime this routine is called, the running
//			checksum is set to zero
// Output - Return Values:
//   	0 = CheckSum OK, No Errors
//		1 = Checksum received is incorrect
//		2 = GetCheckSum Token echoed incorrectly
//		3 = TimeOut while waiting for checksum command and running checksum
//				byte to be echoed
short	GetCheckSum(void);

//==========================================================================
//	CLOSE DEVICE - This function sends the stop scan command to stop a scan
//  if scanning, sends a master reset command, and closes the currently
//  selected comm port.
//
// Input: None
// Other Routines Called: None
// Output: None
// Return Values: None
void	CloseDevice(void);

//==========================================================================
// CHANGE ANALOG OUTPUT - This function instructs the Model 203 to change
//		the output voltage of the selected analog output.
//
// Input:
//		NewVoltage = New voltage for analog output - 0 to 5 Volts
//		DacToSet = 1 or 2
// Global Variables Referenced: MAXCOUNT
// Other Routines Called:
// Output - Global Variables: None
//		M203->dac1volts = voltage of analog output 1 OR
//		M203->dac2volts = voltage of analog output 2
// Output - Return Values:
//   	0 = No Errors -
//    1 = Invalid Analog Output Port
short	ChangeAnalogOutput(short DacToSet);

//==========================================================================
// CheckScanTime - This function changes the Model 203 scan interval. If the
//		new scan interval is within acceptable limits (mintime < scanInterval <
//		maxtime), M203->ScanInterval is updated with NewScanInterval. 	Also,
//		MinTime and MaxTime are updated based current baud rate, wordCnt,
//		etc.
//
// Input:
//		Following variables need to have been initialized within the M203 struct
//			M203->totalPeriods, M203->wordCnt, M203->baud
//		NewScanInterval = the requested scan interval
//
// Other Routines Called: None
// Output: Maxtime, Mintime, M203->scanInterval
// Return Values:
// 	0 = The selected Scan Interval is acceptable
//	 	1 = Scan interval NOT within limits
short	CheckScanTime(double *maxTime, double *minTime);

//==========================================================================
// ECHO TEST - This routine performs a serial byte echo test which can be used
//		to test serial communications.  This routines sends characters 1 through
//		through 255 (not 0x00) to the Model 203 and tests the eched bytes.
//		for each successful byte transferred, received, and compared, M203->
//		echoTestCount is incremented.
//
// Input:
// Other Routines Called: None
// Output:
// Return Values:
//   	0 = Successfull byte echo, no errors
//		1 = Wrong byte echoed
//		2 = Echo Test Timeout (waiting for echo byte)
short	EchoTest(void);

void	Gconv(void);
void	RemFromBuffer(short size);
*/
